/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: _gdsl_mem.h
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: _gdsl_mem.h,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef _GDSL_MEM_H_
#define _GDSL_MEM_H_

#include <stdlib.h>

/*
 * Malloc function: call the malloc function of the module.
 * By default, this function is set to stdlib's malloc().
 * In case of success, returns the newly allocated memory zone.
 * In case of failure, returns NULL.
 */
extern void *
_gdsl_mem_alloc (size_t SIZE
		 );

/*
 * Calloc function: call the calloc function of the module.
 * By default, this function is set to stdlib's calloc().
 * In case of success, returns the newly allocated memory zone.
 * In case of failure, returns NULL.
 */
extern void *
_gdsl_mem_calloc (size_t NMEMB,
		  size_t SIZE
		  );

/*
 * Realloc function: call the realloc function of the module.
 * By default, this function is set to stdlib's realloc().
 * In case of success, returns the newly allocated memory zone.
 * In case of failure, returns NULL.
 */
extern void *
_gdsl_mem_realloc (void *P,
		   size_t SIZE
		   );

/*
 * Module free function: call the free function of the module.
 * By default, this function is set to stdlib's free().
 * Requires: p != NULL
 */
extern void 
_gdsl_mem_free (void *P
		);

/*
 * Change module's functions to user's ones.
 */
extern void 
_gdsl_mem_set_functions (void * (*malloc_user_fun) (size_t),
			 void (*free_user_fun) (void *),
			 void * (*calloc_user_fun) (size_t, size_t),
			 void * (*realloc_user_fun) (void *, size_t)
			 );

/*
 * Reset the default memory functions of the module to stlib's ones:
 * malloc, calloc, realloc, free.
 */
extern void 
_gdsl_mem_set_default_functions (void);

/*
 * Returns the amount of allocated memory if HAVE_MEM_DEBUG is defined.
 * Returns an undefined value otherwise.
 */
extern long int
_gdsl_mem_get_allocated_memory (void);

#endif /* _GDSL_MEM_H_ */
