/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: gdsl_types.h
 * $Revision: 1.3 $
 * $Date: 2002/04/22 16:09:59 $
 * $Author: ndarnis $
 * $Id: gdsl_types.h,v 1.3 2002/04/22 16:09:59 ndarnis Exp $
 */

#ifndef _GDSL_TYPES_H_
#define _GDSL_TYPES_H_

#include <stdio.h>

typedef void *gdsl_element_t;

/*
 * This function type is for allocating a new gdsl_element_t data structure
 * using USER_DATA infos.
 * In case of succes, a function of this type MUST return its allocated object.
 * In case of failure, a function of this type MUST return NULL.
 */
typedef gdsl_element_t (* gdsl_alloc_func_t) (void *USER_DATA);

/*
 * This function type is for freeing a gdsl_element_t E allocated by a function 
 * of type gdsl_alloc_func_t. A function of this type must free the ressources 
 * allocated by the corresponding call to the function of type 
 * gdsl_alloc_func_t. The gdsl functions doesn't check if E != NULL before 
 * calling this function.
 */
typedef void (* gdsl_free_func_t) (gdsl_element_t E);

/*
 *
 */
typedef gdsl_element_t (* gdsl_copy_func_t) (gdsl_element_t E);

/*
 * This function type is for mapping a gdsl_element_t E. The optional non NULL
 * value USER_DATA could be used to do special thing if needed. In gdsl 
 * functions that uses gdsl_map_func_t, if a function of this type returns 0 on 
 * its element it returns to the calling function. It is usefull to break a 
 * mapping.
 */
typedef int (* gdsl_map_func_t) (gdsl_element_t E, 
				 void *USER_DATA);

/*
 * This function type is used to compare two gdsl_element_t data structures.
 * The E argument is always the one in the data structure, VALUE is always the 
 * one the user wants to compare with.
 * A function of that type must return a value V according to the following 
 * rules:
 * V < 0 : if E is assumed to be less than VALUE,
 * V == 0: if E is assumed to be equal to VALUE,
 * V > 0 : if E is assumed to be greather than VALUE,
 */
typedef int (* gdsl_compare_func_t) (gdsl_element_t E, 
				     void *VALUE);

/*
 * This function type is for writing a gdsl_element_t E to OUTPUT_FILE. 
 * Additional USER_DATA could be passed to it.
 */
typedef void (* gdsl_write_func_t) (gdsl_element_t E, 
				    FILE *OUTPUT_FILE, 
				    void *USER_DATA);

#ifdef _NO_LIBGDSL_TYPES_

#else /* _NO_LIBGDSL_TYPES_ */

#include <sys/types.h>

#ifdef HAVE_LINUX

typedef char          byte;
typedef unsigned char ubyte;

#else /* not HAVE_LINUX */

#ifndef _SYS_TYPES_H
typedef unsigned long int ulong;
#endif /* _SYS_TYPES_H */

#endif /* not HAVE_LINUX */

#ifdef TRUE
#undef TRUE
#endif

#ifdef FALSE
#undef FALSE
#endif

#ifdef bool
#undef bool
#endif

typedef enum { FALSE = 0, TRUE = 1 } bool;

#endif /* not _NO_LIBGDSL_TYPES_ */

#endif /* _GDSL_TYPES_H_ */
